@@ -188,6 +188,7 @@ namespace libtorrent
 		//	get_download_queue() is called, it will be invalidated.
 		block_info* blocks;
 
+#ifndef TORRENT_NO_DEPRECATE
 		// the speed classes. These may be used by the piece picker to
 		// coalesce requests of similar download rates
 		enum state_t { none, slow, medium, fast };
@@ -204,7 +205,12 @@ namespace libtorrent
 		// downloaded pieces down. Pieces set to ``none`` can be converted into
 		// any of ``fast``, ``medium`` or ``slow`` as soon as a peer want to
 		// download from it.
-		state_t piece_state;
+		state_t TORRENT_DEPRECATED_MEMBER piece_state;
+#else
+		// hidden
+		enum deprecated_state_t { none, slow, medium, fast };
+		deprecated_state_t deprecated_piece_state;
+#endif
 	};
 
 	// for boost::hash (and to support using this type in unordered_map etc.)
@@ -369,14 +375,14 @@ namespace libtorrent
 		// deadline (and flags) of a piece can be changed by calling this
 		// function again.
 		// 
-		// The ``flags`` parameter can be used to ask libtorrent to send an alert
+		// The ``flags`` parameter can be used to ask libtorrent to post an alert
 		// once the piece has been downloaded, by passing alert_when_available.
 		// When set, the read_piece_alert alert will be delivered, with the piece
 		// data, when it's downloaded.
 		// 
 		// If the piece is already downloaded when this call is made, nothing
 		// happens, unless the alert_when_available flag is set, in which case it
-		// will do the same thing as calling read_piece() for ``index``.
+		// will have the same effect as calling read_piece() for ``index``.
 		// 
 		// ``deadline`` is the number of milliseconds until this piece should be
 		// completed.
@@ -408,7 +414,7 @@ namespace libtorrent
 		// priority, but the priority is used as a weight.
 		// 
 		// Peers whose Torrent has a higher priority will take precedence when
-		// distributing unchoke slots. This is a strict prioritization where
+		// distributing unchoke slots. This is a strict prioritisation where
 		// every interested peer on a high priority torrent will be unchoked
 		// before any other, lower priority, torrents have any peers unchoked.
 		void set_priority(int prio) const;
@@ -494,7 +500,7 @@ namespace libtorrent
 		// pieces from it, unless it's paused, queued, checking or seeding.
 		// ``remove_url_seed()`` removes the given url if it exists already.
 		// ``url_seeds()`` return a set of the url seeds currently in this
-		// torrent. Note that urls that fails may be removed automatically from
+		// torrent. Note that URLs that fails may be removed automatically from
 		// the list.
 		// 
 		// See http-seeding_ for more information.
@@ -562,7 +568,7 @@ namespace libtorrent
 		// .. note::
 		// 	Torrents that are auto-managed may be automatically resumed again. It
 		// 	does not make sense to pause an auto-managed torrent without making it
-		// 	not automanaged first. Torrents are auto-managed by default when added
+		// 	not auto-managed first. Torrents are auto-managed by default when added
 		// 	to the session. For more information, see queuing_.
 		// 
 		void pause(int flags = 0) const;
@@ -880,7 +886,7 @@ namespace libtorrent
 		// 
 		// Note that when a torrent first starts up, and it needs a certificate,
 		// it will suspend connecting to any peers until it has one. It's
-		// typically desirable to resume the torrent after setting the ssl
+		// typically desirable to resume the torrent after setting the SSL
 		// certificate.
 		// 
 		// If you receive a torrent_need_cert_alert, you need to call this to
@@ -1134,7 +1140,7 @@ namespace libtorrent
 		// Magnet links, and other torrents that start out without having
 		// metadata are pinned automatically. This is to give the client a chance
 		// to get the metadata and save it before it's unloaded. In this case, it
-		// may be useful to un-pin the torrent once its metadata has been saved
+		// may be useful to unpin the torrent once its metadata has been saved
 		// to disk.
 		// 
 		// For more information about dynamically loading and unloading torrents,
@@ -1158,7 +1164,7 @@ namespace libtorrent
 		// can be done by using this other than an unnecessary connection attempt
 		// is made. If the torrent is uninitialized or in queued or checking
 		// mode, this will throw libtorrent_exception. The second (optional)
-		// argument will be bitwised ORed into the source mask of this peer.
+		// argument will be bitwise ORed into the source mask of this peer.
 		// Typically this is one of the source flags in peer_info. i.e.
 		// ``tracker``, ``pex``, ``dht`` etc.
 		//
@@ -1172,7 +1178,7 @@ namespace libtorrent
 		// 0x04 supports uTP. If this is not set, the peer will only be contacted
 		//      over TCP.
 		// 
-		// 0x08 supports holepunching protocol. If this
+		// 0x08 supports hole punching protocol. If this
 		//      flag is received from a peer, it can be
 		//      used as a rendezvous point in case direct
 		//      connections to the peer fail
@@ -1252,7 +1258,7 @@ namespace libtorrent
 		// directory, if there is one. The source files will still be removed in
 		// that case.
 		// 
-		// Files that have been renamed to have absolute pahts are not moved by
+		// Files that have been renamed to have absolute paths are not moved by
 		// this function. Keep in mind that files that don't belong to the
 		// torrent but are stored in the torrent's directory may be moved as
 		// well. This goes for files that have been renamed to absolute paths
