@@ -37,22 +37,18 @@ POSSIBILITY OF SUCH DAMAGE.
 #include "libtorrent/error_code.hpp"
 #include "libtorrent/broadcast_socket.hpp"
 #include "libtorrent/http_connection.hpp"
-#include "libtorrent/connection_queue.hpp"
-#include "libtorrent/intrusive_ptr_base.hpp"
 #include "libtorrent/thread.hpp"
 #include "libtorrent/deadline_timer.hpp"
+#include "libtorrent/enum_net.hpp"
+#include "libtorrent/resolver.hpp"
 
 #include <boost/function/function1.hpp>
-#include <boost/function/function4.hpp>
+#include <boost/function/function5.hpp>
 #include <boost/noncopyable.hpp>
 #include <boost/shared_ptr.hpp>
+#include <boost/enable_shared_from_this.hpp>
 #include <set>
 
-
-#if defined(TORRENT_UPNP_LOGGING)
-#include <fstream>
-#endif
-
 namespace libtorrent
 {
 
@@ -97,56 +93,107 @@ namespace libtorrent
 	}
 
 	// the boost.system error category for UPnP errors
-	TORRENT_EXPORT boost::system::error_category& get_upnp_category();
+	TORRENT_EXPORT boost::system::error_category& upnp_category();
+
+#ifndef TORRENT_NO_DEPRECATED
+	TORRENT_DEPRECATED TORRENT_EXPORT
+	boost::system::error_category& get_upnp_category();
+#endif
 
 // int: port-mapping index
 // address: external address as queried from router
 // int: external port
+// int: protocol (UDP, TCP)
 // std::string: error message
 // an empty string as error means success
-// a port-mapping index of -1 means it's
-// an informational log message
-typedef boost::function<void(int, address, int, error_code const&)> portmap_callback_t;
+typedef boost::function<void(int, address, int, int, error_code const&)> portmap_callback_t;
 typedef boost::function<void(char const*)> log_callback_t;
 
+struct parse_state
+{
+	parse_state(): in_service(false) {}
+	bool in_service;
+	std::list<std::string> tag_stack;
+	std::string control_url;
+	std::string service_type;
+	std::string model;
+	std::string url_base;
+	bool top_tags(const char* str1, const char* str2)
+	{
+		std::list<std::string>::reverse_iterator i = tag_stack.rbegin();
+		if (i == tag_stack.rend()) return false;
+		if (!string_equal_no_case(i->c_str(), str2)) return false;
+		++i;
+		if (i == tag_stack.rend()) return false;
+		if (!string_equal_no_case(i->c_str(), str1)) return false;
+		return true;
+	}
+};
+
+struct error_code_parse_state
+{
+	error_code_parse_state(): in_error_code(false), exit(false), error_code(-1) {}
+	bool in_error_code;
+	bool exit;
+	int error_code;
+};
+
+struct ip_address_parse_state: error_code_parse_state
+{
+	ip_address_parse_state(): in_ip_address(false) {}
+	bool in_ip_address;
+	std::string ip_address;
+};
+
+TORRENT_EXTRA_EXPORT void find_control_url(int type, char const* string
+	, int str_len, parse_state& state);
+
+TORRENT_EXTRA_EXPORT void find_error_code(int type, char const* string
+	, int str_len, error_code_parse_state& state);
+
+TORRENT_EXTRA_EXPORT void find_ip_address(int type, char const* string
+	, int str_len, ip_address_parse_state& state);
+
 // TODO: support using the windows API for UPnP operations as well
-class TORRENT_EXTRA_EXPORT upnp : public intrusive_ptr_base<upnp>
+class TORRENT_EXTRA_EXPORT upnp : public boost::enable_shared_from_this<upnp>
 {
 public:
-	upnp(io_service& ios, connection_queue& cc
+	upnp(io_service& ios
 		, address const& listen_interface, std::string const& user_agent
 		, portmap_callback_t const& cb, log_callback_t const& lcb
-		, bool ignore_nonrouters, void* state = 0);
+		, bool ignore_nonrouters);
 	~upnp();
 
-	void* drain_state();
+	void set_user_agent(std::string const& v) { m_user_agent = v; }
+
+	void start();
 
 	enum protocol_type { none = 0, udp = 1, tcp = 2 };
 
 	// Attempts to add a port mapping for the specified protocol. Valid protocols are
 	// ``upnp::tcp`` and ``upnp::udp`` for the UPnP class and ``natpmp::tcp`` and
 	// ``natpmp::udp`` for the NAT-PMP class.
-	// 
+	//
 	// ``external_port`` is the port on the external address that will be mapped. This
 	// is a hint, you are not guaranteed that this port will be available, and it may
 	// end up being something else. In the portmap_alert_ notification, the actual
 	// external port is reported.
-	// 
+	//
 	// ``local_port`` is the port in the local machine that the mapping should forward
 	// to.
-	// 
+	//
 	// The return value is an index that identifies this port mapping. This is used
 	// to refer to mappings that fails or succeeds in the portmap_error_alert_ and
 	// portmap_alert_ respectively. If The mapping fails immediately, the return value
 	// is -1, which means failure. There will not be any error alert notification for
 	// mappings that fail with a -1 return value.
-	int add_mapping(protocol_type p, int external_port, int local_port);
+	int add_mapping(protocol_type p, int external_port, tcp::endpoint local_ep);
 
 	// This function removes a port mapping. ``mapping_index`` is the index that refers
 	// to the mapping you want to remove, which was returned from add_mapping().
 	void delete_mapping(int mapping_index);
 
-	bool get_mapping(int mapping_index, int& local_port, int& external_port, int& protocol) const;
+	bool get_mapping(int mapping_index, tcp::endpoint& local_ep, int& external_port, int& protocol) const;
 
 	void discover_device();
 	void close();
@@ -161,6 +208,8 @@ public:
 
 private:
 
+	boost::shared_ptr<upnp> self() { return shared_from_this(); }
+
 	void map_timer(error_code const& ec);
 	void try_map_upnp(mutex::scoped_lock& l, bool timer = false);
 	void discover_device_impl(mutex::scoped_lock& l);
@@ -172,7 +221,7 @@ private:
 	// safer to always assume that we have to ask for
 	// permanent leases
 	enum { default_lease_time = 0 };
-	
+
 	void resend_request(error_code const& e);
 	void on_reply(udp::endpoint const& from, char* buffer
 		, std::size_t bytes_transferred);
@@ -181,7 +230,6 @@ private:
 	void next(rootdevice& d, int i, mutex::scoped_lock& l);
 	void update_map(rootdevice& d, int i, mutex::scoped_lock& l);
 
-	
 	void on_upnp_xml(error_code const& e
 		, libtorrent::http_parser const& p, rootdevice& d
 		, http_connection& c);
@@ -213,11 +261,10 @@ private:
 		global_mapping_t()
 			: protocol(none)
 			, external_port(0)
-			, local_port(0)
 		{}
 		int protocol;
 		int external_port;
-		int local_port;
+		tcp::endpoint local_ep;
 	};
 
 	struct mapping_t
@@ -225,20 +272,19 @@ private:
 		enum action_t { action_none, action_add, action_delete };
 		mapping_t()
 			: action(action_none)
-			, local_port(0)
 			, external_port(0)
 			, protocol(none)
 			, failcount(0)
 		{}
 
 		// the time the port mapping will expire
-		ptime expires;
-		
-		int action;
+		time_point expires;
 
-		// the local port for this mapping. If this is set
+		// the local port for this mapping. The port is set
 		// to 0, the mapping is not in use
-		int local_port;
+		tcp::endpoint local_ep;
+
+		int action;
 
 		// the external (on the NAT router) port
 		// for the mapping. This is the port we
@@ -254,8 +300,7 @@ private:
 
 	struct rootdevice
 	{
-		rootdevice(): service_namespace(0)
-			, port(0)
+		rootdevice(): port(0)
 			, lease_duration(default_lease_time)
 			, supports_specific_external(true)
 			, disabled(false)
@@ -272,19 +317,23 @@ private:
 			TORRENT_ASSERT(magic == 1337);
 			magic = 0;
 		}
+#if __cplusplus >= 201103L
+		rootdevice(rootdevice const&) = default;
+		rootdevice& operator=(rootdevice const&) = default;
+#endif
 #endif
 
 		// the interface url, through which the list of
 		// supported interfaces are fetched
 		std::string url;
-	
+
 		// the url to the WANIP or WANPPP interface
 		std::string control_url;
 		// either the WANIP namespace or the WANPPP namespace
-		char const* service_namespace;
+		std::string service_namespace;
 
 		std::vector<mapping_t> mapping;
-		
+
 		// this is the hostname, port and path
 		// component of the url or the control_url
 		// if it has been found
@@ -297,7 +346,7 @@ private:
 		// true if the device supports specifying a
 		// specific external port, false if it doesn't
 		bool supports_specific_external;
-		
+
 		bool disabled;
 
 		// this is true if the IP of this device is not
@@ -319,11 +368,11 @@ private:
 			upnp_connection->close();
 			upnp_connection.reset();
 		}
-		
+
 		bool operator<(rootdevice const& rhs) const
 		{ return url < rhs.url; }
 	};
-	
+
 	struct upnp_state_t
 	{
 		std::vector<global_mapping_t> mappings;
@@ -332,11 +381,11 @@ private:
 
 	std::vector<global_mapping_t> m_mappings;
 
-	std::string const& m_user_agent;
-	
+	std::string m_user_agent;
+
 	// the set of devices we've found
 	std::set<rootdevice> m_devices;
-	
+
 	portmap_callback_t m_callback;
 	log_callback_t m_log_callback;
 
@@ -345,6 +394,8 @@ private:
 
 	io_service& m_io_service;
 
+	resolver m_resolver;
+
 	// the udp socket used to send and receive
 	// multicast messages on the network
 	broadcast_socket m_socket;
@@ -362,32 +413,28 @@ private:
 	// response from a router UPnP device, we override the ignoring behavior and
 	// map them anyway.
 	deadline_timer m_map_timer;
-	
+
 	bool m_disabled;
 	bool m_closing;
 	bool m_ignore_non_routers;
 
-	connection_queue& m_cc;
-
 	mutex m_mutex;
 
 	std::string m_model;
+
+	// cache of interfaces
+	mutable std::vector<ip_interface> m_interfaces;
+	mutable time_point m_last_if_update;
 };
 
 }
 
-#if BOOST_VERSION >= 103500
-
 namespace boost { namespace system {
 
 	template<> struct is_error_code_enum<libtorrent::upnp_errors::error_code_enum>
 	{ static const bool value = true; };
 
-	template<> struct is_error_condition_enum<libtorrent::upnp_errors::error_code_enum>
-	{ static const bool value = true; };
 } }
 
-#endif // BOOST_VERSION
-
 #endif
 
